@@ -18,6 +18,7 @@ import (
 	"os"
 	"path/filepath"
 	"reflect"
+	"runtime"
 	"strings"
 	"testing"
 	"time"
@@ -101,77 +102,6 @@ func (t *tokenRequestCountingPolicy) Do(req *policy.Request) (*http.Response, er
 	return req.Next()
 }
 
-func TestResponseErrors(t *testing.T) {
-	// compact removes whitespace from errors to simplify validation
-	compact := func(s string) string {
-		return strings.Map(func(r rune) rune {
-			if r == ' ' || r == '\n' || r == '\t' {
-				return -1
-			}
-			return r
-		}, s)
-	}
-	content := "no tokens here"
-	statusCode := http.StatusTeapot
-	validate := func(t *testing.T, err error) {
-		require.Error(t, err)
-		flatErr := compact(err.Error())
-		actual := strings.Count(flatErr, compact(http.StatusText(statusCode)))
-		require.Equal(t, 1, actual, "error message should include response exactly once:\n%s", err.Error())
-		actual = strings.Count(flatErr, compact(content))
-		require.Equal(t, 1, actual, "error message should include body exactly once:\n%s", err.Error())
-	}
-
-	for _, client := range []struct {
-		name string
-		ctor func(co policy.ClientOptions) (azcore.TokenCredential, error)
-	}{
-		{
-			name: "confidential",
-			ctor: func(co policy.ClientOptions) (azcore.TokenCredential, error) {
-				return NewClientSecretCredential(fakeTenantID, fakeClientID, fakeSecret, &ClientSecretCredentialOptions{ClientOptions: co})
-			},
-		},
-		{
-			name: "managed identity",
-			ctor: func(co policy.ClientOptions) (azcore.TokenCredential, error) {
-				return NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{ClientOptions: co})
-			},
-		},
-		{
-			name: "public",
-			ctor: func(co policy.ClientOptions) (azcore.TokenCredential, error) {
-				return NewUsernamePasswordCredential(fakeTenantID, fakeClientID, "username", "password", &UsernamePasswordCredentialOptions{ClientOptions: co})
-			},
-		},
-	} {
-		t.Run(client.name, func(t *testing.T) {
-			cred, err := client.ctor(policy.ClientOptions{
-				Retry: policy.RetryOptions{MaxRetries: -1},
-				Transport: &mockSTS{
-					tokenRequestCallback: func(*http.Request) *http.Response {
-						return &http.Response{
-							Body:       io.NopCloser(bytes.NewBufferString(content)),
-							Status:     http.StatusText(statusCode),
-							StatusCode: statusCode,
-						}
-					},
-				},
-			})
-			require.NoError(t, err)
-			_, err = cred.GetToken(ctx, testTRO)
-			validate(t, err)
-
-			t.Run("ChainedTokenCredential", func(t *testing.T) {
-				chain, err := NewChainedTokenCredential([]azcore.TokenCredential{cred}, nil)
-				require.NoError(t, err)
-				_, err = chain.GetToken(ctx, testTRO)
-				validate(t, err)
-			})
-		})
-	}
-}
-
 func TestTenantID(t *testing.T) {
 	type tc struct {
 		name           string
@@ -203,12 +133,6 @@ func TestTenantID(t *testing.T) {
 			},
 			tenantOptional: true,
 		},
-		{
-			name: credNameAzurePipelines,
-			ctor: func(tenant string) (azcore.TokenCredential, error) {
-				return NewAzurePipelinesCredential(tenant, fakeClientID, "service-connection", tokenValue, nil)
-			},
-		},
 		{
 			name: credNameBrowser,
 			ctor: func(tenant string) (azcore.TokenCredential, error) {
@@ -283,49 +207,38 @@ func TestTenantID(t *testing.T) {
 	}
 }
 
-type testCache []byte
-
-func (c *testCache) Export(_ context.Context, m cache.Marshaler, _ cache.ExportHints) (err error) {
-	*c, err = m.Marshal()
-	return
-}
-
-func (c *testCache) Replace(_ context.Context, u cache.Unmarshaler, _ cache.ReplaceHints) error {
-	return u.Unmarshal(*c)
-}
-
 func TestUserAuthentication(t *testing.T) {
 	type authenticater interface {
 		azcore.TokenCredential
-		Authenticate(context.Context, *policy.TokenRequestOptions) (AuthenticationRecord, error)
+		authenticate(context.Context, *policy.TokenRequestOptions) (authenticationRecord, error)
 	}
 	for _, credential := range []struct {
 		name                    string
 		interactive, recordable bool
-		new                     func(Cache, azcore.ClientOptions, AuthenticationRecord, bool) (authenticater, error)
+		new                     func(*tokenCachePersistenceOptions, azcore.ClientOptions, authenticationRecord, bool) (authenticater, error)
 	}{
 		{
 			name: credNameBrowser,
-			new: func(c Cache, co azcore.ClientOptions, ar AuthenticationRecord, disableAutoAuth bool) (authenticater, error) {
+			new: func(tcpo *tokenCachePersistenceOptions, co azcore.ClientOptions, ar authenticationRecord, disableAutoAuth bool) (authenticater, error) {
 				return NewInteractiveBrowserCredential(&InteractiveBrowserCredentialOptions{
 					AdditionallyAllowedTenants:     []string{"*"},
-					AuthenticationRecord:           ar,
-					Cache:                          c,
+					authenticationRecord:           ar,
 					ClientOptions:                  co,
-					DisableAutomaticAuthentication: disableAutoAuth,
+					disableAutomaticAuthentication: disableAutoAuth,
+					tokenCachePersistenceOptions:   tcpo,
 				})
 			},
 			interactive: true,
 		},
 		{
 			name: credNameDeviceCode,
-			new: func(c Cache, co azcore.ClientOptions, ar AuthenticationRecord, disableAutoAuth bool) (authenticater, error) {
+			new: func(tcpo *tokenCachePersistenceOptions, co azcore.ClientOptions, ar authenticationRecord, disableAutoAuth bool) (authenticater, error) {
 				o := DeviceCodeCredentialOptions{
 					AdditionallyAllowedTenants:     []string{"*"},
-					AuthenticationRecord:           ar,
-					Cache:                          c,
+					authenticationRecord:           ar,
 					ClientOptions:                  co,
-					DisableAutomaticAuthentication: disableAutoAuth,
+					disableAutomaticAuthentication: disableAutoAuth,
+					tokenCachePersistenceOptions:   tcpo,
 				}
 				if recording.GetRecordMode() == recording.PlaybackMode {
 					o.UserPrompt = func(context.Context, DeviceCodeMessage) error { return nil }
@@ -337,12 +250,12 @@ func TestUserAuthentication(t *testing.T) {
 		},
 		{
 			name: credNameUserPassword,
-			new: func(c Cache, co azcore.ClientOptions, ar AuthenticationRecord, disableAutoAuth bool) (authenticater, error) {
+			new: func(tcpo *tokenCachePersistenceOptions, co azcore.ClientOptions, ar authenticationRecord, disableAutoAuth bool) (authenticater, error) {
 				opts := UsernamePasswordCredentialOptions{
-					AdditionallyAllowedTenants: []string{"*"},
-					AuthenticationRecord:       ar,
-					Cache:                      c,
-					ClientOptions:              co,
+					AdditionallyAllowedTenants:   []string{"*"},
+					authenticationRecord:         ar,
+					ClientOptions:                co,
+					tokenCachePersistenceOptions: tcpo,
 				}
 				return NewUsernamePasswordCredential(liveUser.tenantID, developerSignOnClientID, liveUser.username, liveUser.password, &opts)
 			},
@@ -367,15 +280,15 @@ func TestUserAuthentication(t *testing.T) {
 				}}
 
 				co := azcore.ClientOptions{Cloud: cc, Transport: &sts}
-				cred, err := credential.new(Cache{}, co, AuthenticationRecord{}, false)
+				cred, err := credential.new(nil, co, authenticationRecord{}, false)
 				require.NoError(t, err)
-				_, err = cred.Authenticate(context.Background(), nil)
+				_, err = cred.authenticate(context.Background(), nil)
 				require.NoError(t, err)
 
 				t.Setenv(azureAuthorityHost, cc.ActiveDirectoryAuthorityHost)
-				cred, err = credential.new(Cache{}, azcore.ClientOptions{Transport: &sts}, AuthenticationRecord{}, false)
+				cred, err = credential.new(nil, azcore.ClientOptions{Transport: &sts}, authenticationRecord{}, false)
 				require.NoError(t, err)
-				_, err = cred.Authenticate(context.Background(), nil)
+				_, err = cred.authenticate(context.Background(), nil)
 				if cc.ActiveDirectoryAuthorityHost == customCloud.ActiveDirectoryAuthorityHost {
 					// Authenticate should return an error because it can't map an unknown host to a default scope
 					require.ErrorIs(t, err, errScopeRequired)
@@ -401,14 +314,14 @@ func TestUserAuthentication(t *testing.T) {
 			counter := tokenRequestCountingPolicy{}
 			co.PerCallPolicies = append(co.PerCallPolicies, &counter)
 
-			cred, err := credential.new(Cache{}, co, AuthenticationRecord{}, false)
+			cred, err := credential.new(nil, co, authenticationRecord{}, false)
 			require.NoError(t, err)
-			ar, err := cred.Authenticate(context.Background(), &testTRO)
+			ar, err := cred.authenticate(context.Background(), &testTRO)
 			require.NoError(t, err)
 
-			// some fields of the returned AuthenticationRecord should have specific values
-			require.Equal(t, developerSignOnClientID, ar.ClientID)
-			require.Equal(t, supportedAuthRecordVersions[0], ar.Version)
+			// some fields of the returned authenticationRecord should have specific values
+			require.Equal(t, ar.ClientID, developerSignOnClientID)
+			require.Equal(t, ar.Version, supportedAuthRecordVersions[0])
 			// all others should have nonempty values
 			v := reflect.Indirect(reflect.ValueOf(&ar))
 			for _, f := range reflect.VisibleFields(reflect.TypeOf(ar)) {
@@ -418,47 +331,49 @@ func TestUserAuthentication(t *testing.T) {
 			require.Equal(t, 1, counter.count)
 		})
 
-		t.Run("PersistentCache/"+credential.name, func(t *testing.T) {
-			if credential.name == credNameBrowser && !runManualTests {
-				t.Skipf("set %s to run this test", azidentityRunManualTests)
+		t.Run("PersistentCache_Live/"+credential.name, func(t *testing.T) {
+			t.Skip("TODO: fix this test after restoring persistent cache feature")
+			switch recording.GetRecordMode() {
+			case recording.LiveMode:
+				if credential.interactive && !runManualTests {
+					t.Skipf("set %s to run this test", azidentityRunManualTests)
+				}
+			case recording.PlaybackMode, recording.RecordingMode:
+				if !credential.recordable {
+					t.Skip("this test can't be recorded")
+				}
 			}
-			tokenReqs := 0
-			c := internal.NewCache(func(bool) (cache.ExportReplace, error) {
-				return &testCache{}, nil
-			})
-			co := azcore.ClientOptions{Transport: &mockSTS{
-				tokenRequestCallback: func(*http.Request) *http.Response {
-					tokenReqs++
-					return nil
-				},
-			}}
+			if runtime.GOOS != "windows" {
+				t.Skip("this test runs only on Windows")
+			}
+			p, err := internal.CacheFilePath(t.Name())
+			require.NoError(t, err)
+			os.Remove(p)
+			co, stop := initRecording(t)
+			defer stop()
+			counter := tokenRequestCountingPolicy{}
+			co.PerCallPolicies = append(co.PerCallPolicies, &counter)
+			tcpo := tokenCachePersistenceOptions{Name: t.Name()}
 
-			cred, err := credential.new(c, co, AuthenticationRecord{}, false)
+			cred, err := credential.new(&tcpo, co, authenticationRecord{}, true)
 			require.NoError(t, err)
-			record, err := cred.Authenticate(ctx, &testTRO)
+			record, err := cred.authenticate(context.Background(), &testTRO)
 			require.NoError(t, err)
-			_, err = cred.GetToken(ctx, testTRO)
+			defer os.Remove(p)
+			tk, err := cred.GetToken(context.Background(), testTRO)
 			require.NoError(t, err)
-			require.Equal(t, 1, tokenReqs)
+			require.Equal(t, 1, counter.count)
 
-			// cred2 should return the token cached by cred
-			cred2, err := credential.new(c, co, record, true)
+			cred2, err := credential.new(&tcpo, co, record, true)
 			require.NoError(t, err)
-			_, err = cred2.GetToken(ctx, testTRO)
+			tk2, err := cred2.GetToken(context.Background(), testTRO)
 			require.NoError(t, err)
-			require.Equal(t, 1, tokenReqs)
-
-			// cred should request a new token because the cached one isn't a CAE token
-			caeTRO := testTRO
-			caeTRO.EnableCAE = true
-			_, err = cred.GetToken(ctx, caeTRO)
-			require.NoError(t, err)
-			require.Equal(t, 2, tokenReqs)
+			require.Equal(t, tk.Token, tk2.Token)
 		})
 
 		if credential.interactive {
 			t.Run("DisableAutomaticAuthentication/"+credential.name, func(t *testing.T) {
-				cred, err := credential.new(Cache{}, policy.ClientOptions{Transport: &mockSTS{}}, AuthenticationRecord{}, true)
+				cred, err := credential.new(nil, policy.ClientOptions{Transport: &mockSTS{}}, authenticationRecord{}, true)
 				require.NoError(t, err)
 				expected := policy.TokenRequestOptions{
 					Claims:    "claims",
@@ -469,12 +384,12 @@ func TestUserAuthentication(t *testing.T) {
 				_, err = cred.GetToken(context.Background(), expected)
 				require.Contains(t, err.Error(), credential.name)
 				require.Contains(t, err.Error(), "Call Authenticate")
-				var actual *AuthenticationRequiredError
+				var actual *authenticationRequiredError
 				require.ErrorAs(t, err, &actual)
 				require.Equal(t, expected, actual.TokenRequestOptions)
 
 				if credential.name != credNameBrowser || runManualTests {
-					_, err = cred.Authenticate(context.Background(), &testTRO)
+					_, err = cred.authenticate(context.Background(), &testTRO)
 					require.NoError(t, err)
 					// silent auth should succeed this time
 					_, err = cred.GetToken(context.Background(), testTRO)
@@ -482,7 +397,7 @@ func TestUserAuthentication(t *testing.T) {
 				}
 			})
 			t.Run("DisableAutomaticAuthentication/ChainedTokenCredential/"+credential.name, func(t *testing.T) {
-				cred, err := credential.new(Cache{}, policy.ClientOptions{}, AuthenticationRecord{}, true)
+				cred, err := credential.new(nil, policy.ClientOptions{}, authenticationRecord{}, true)
 				require.NoError(t, err)
 				expected := azcore.AccessToken{ExpiresOn: time.Now().UTC(), Token: tokenValue}
 				fake := NewFakeCredential()
@@ -490,7 +405,7 @@ func TestUserAuthentication(t *testing.T) {
 				chain, err := NewChainedTokenCredential([]azcore.TokenCredential{cred, fake}, nil)
 				require.NoError(t, err)
 				// ChainedTokenCredential should continue iterating when a credential returns
-				// AuthenticationRequiredError i.e., it should call fake.GetToken() and return the expected token
+				// authenticationRequiredError i.e., it should call fake.GetToken() and return the expected token
 				actual, err := chain.GetToken(context.Background(), testTRO)
 				require.NoError(t, err)
 				require.Equal(t, expected, actual)
@@ -688,14 +603,6 @@ func TestAdditionallyAllowedTenants(t *testing.T) {
 					return NewAzureDeveloperCLICredential(&o)
 				},
 			},
-			{
-				name: credNameAzurePipelines,
-				ctor: func(co azcore.ClientOptions) (azcore.TokenCredential, error) {
-					o := AzurePipelinesCredentialOptions{AdditionallyAllowedTenants: test.allowed, ClientOptions: co}
-					return NewAzurePipelinesCredential(fakeTenantID, fakeClientID, "service-connection", tokenValue, &o)
-				},
-				env: map[string]string{systemOIDCRequestURI: "https://localhost/instance"},
-			},
 			{
 				name: credNameCert,
 				ctor: func(co azcore.ClientOptions) (azcore.TokenCredential, error) {
@@ -872,7 +779,7 @@ func TestAdditionallyAllowedTenants(t *testing.T) {
 			c, err := NewInteractiveBrowserCredential(&InteractiveBrowserCredentialOptions{
 				AdditionallyAllowedTenants: test.allowed,
 				// this enables testing the credential's tenant resolution without having to authenticate
-				DisableAutomaticAuthentication: true,
+				disableAutomaticAuthentication: true,
 			})
 			require.NoError(t, err)
 			_, err = c.GetToken(context.Background(), tro)
@@ -883,7 +790,7 @@ func TestAdditionallyAllowedTenants(t *testing.T) {
 				// tenant resolution should have succeeded because the specified tenant is allowed,
 				// however the credential should have returned a different error because automatic
 				// authentication is disabled
-				var e *AuthenticationRequiredError
+				var e *authenticationRequiredError
 				require.ErrorAs(t, err, &e)
 			}
 		})
@@ -953,7 +860,6 @@ func TestClaims(t *testing.T) {
 	claim := `"test":"pass"`
 	for _, test := range []struct {
 		ctor func(azcore.ClientOptions) (azcore.TokenCredential, error)
-		env  map[string]string
 		name string
 	}{
 		{
@@ -963,14 +869,6 @@ func TestClaims(t *testing.T) {
 				return NewClientAssertionCredential(fakeTenantID, fakeClientID, func(context.Context) (string, error) { return "...", nil }, &o)
 			},
 		},
-		{
-			name: credNameAzurePipelines,
-			ctor: func(co azcore.ClientOptions) (azcore.TokenCredential, error) {
-				o := AzurePipelinesCredentialOptions{ClientOptions: co}
-				return NewAzurePipelinesCredential(fakeTenantID, fakeClientID, "service-connection", tokenValue, &o)
-			},
-			env: map[string]string{systemOIDCRequestURI: "https://localhost/foo/UserRealm/foo"},
-		},
 		{
 			name: credNameCert,
 			ctor: func(co azcore.ClientOptions) (azcore.TokenCredential, error) {
@@ -1027,9 +925,6 @@ func TestClaims(t *testing.T) {
 				name += " CAE"
 			}
 			t.Run(name, func(t *testing.T) {
-				for k, v := range test.env {
-					t.Setenv(k, v)
-				}
 				reqs := 0
 				sts := mockSTS{
 					tokenRequestCallback: func(r *http.Request) *http.Response {
@@ -1183,90 +1078,107 @@ func TestResolveTenant(t *testing.T) {
 	}
 }
 
-func TestConfidentialClientPersistentCache(t *testing.T) {
-	// for WorkloadIdentityCredential
-	tfp := filepath.Join(t.TempDir(), "tokenfile")
-	require.NoError(t, os.WriteFile(tfp, []byte("token"), 0600))
-	for _, credential := range []struct {
-		name string
-		new  func(azcore.ClientOptions, Cache) (azcore.TokenCredential, error)
+func TestTokenCachePersistenceOptions(t *testing.T) {
+	af := filepath.Join(t.TempDir(), t.Name()+credNameWorkloadIdentity)
+	if err := os.WriteFile(af, []byte("assertion"), os.ModePerm); err != nil {
+		t.Fatal(err)
+	}
+	before := internal.NewCache
+	t.Cleanup(func() { internal.NewCache = before })
+	for _, test := range []struct {
+		desc    string
+		options *tokenCachePersistenceOptions
+		err     error
 	}{
 		{
-			name: credNameAssertion,
-			new: func(co azcore.ClientOptions, c Cache) (azcore.TokenCredential, error) {
-				o := ClientAssertionCredentialOptions{Cache: c, ClientOptions: co}
-				return NewClientAssertionCredential(fakeTenantID, fakeClientID, func(context.Context) (string, error) { return "...", nil }, &o)
-			},
-		},
-		// TODO: set SYSTEM_OIDC_REQUEST_URI, fake response
-		// {
-		// 	name: credNameAzurePipelines,
-		// 	new: func(co azcore.ClientOptions, c Cache) (azcore.TokenCredential, error) {
-		// 		o := AzurePipelinesCredentialOptions{Cache: c, ClientOptions: co}
-		// 		return NewAzurePipelinesCredential(fakeTenantID, fakeClientID, "service-connection", tokenValue, &o)
-		// 	},
-		// },
-		{
-			name: credNameCert,
-			new: func(co azcore.ClientOptions, c Cache) (azcore.TokenCredential, error) {
-				o := ClientCertificateCredentialOptions{Cache: c, ClientOptions: co}
-				return NewClientCertificateCredential(fakeTenantID, fakeClientID, allCertTests[0].certs, allCertTests[0].key, &o)
-			},
+			desc: "nil options",
 		},
 		{
-			name: credNameSecret,
-			new: func(co azcore.ClientOptions, c Cache) (azcore.TokenCredential, error) {
-				o := ClientSecretCredentialOptions{Cache: c, ClientOptions: co}
-				return NewClientSecretCredential(fakeTenantID, fakeClientID, fakeSecret, &o)
-			},
+			desc:    "default options",
+			options: &tokenCachePersistenceOptions{},
 		},
 		{
-			name: credNameWorkloadIdentity,
-			new: func(co azcore.ClientOptions, c Cache) (azcore.TokenCredential, error) {
-				o := WorkloadIdentityCredentialOptions{
-					Cache:         c,
-					ClientID:      fakeClientID,
-					ClientOptions: co,
-					TenantID:      fakeTenantID,
-					TokenFilePath: tfp,
-				}
-				return NewWorkloadIdentityCredential(&o)
-			},
+			desc:    "all options set",
+			options: &tokenCachePersistenceOptions{AllowUnencryptedStorage: true, Name: "name"},
 		},
 	} {
-		t.Run(credential.name, func(t *testing.T) {
-			tokenReqs := 0
-			c := internal.NewCache(func(bool) (cache.ExportReplace, error) {
-				return &testCache{}, nil
-			})
-			sts := mockSTS{
-				tokenRequestCallback: func(*http.Request) *http.Response {
-					tokenReqs++
-					return nil
-				},
+		internal.NewCache = func(o *internal.TokenCachePersistenceOptions, _ bool) (cache.ExportReplace, error) {
+			if (test.options == nil) != (o == nil) {
+				t.Fatalf("expected %v, got %v", test.options, o)
 			}
-			cred, err := credential.new(policy.ClientOptions{Transport: &sts}, c)
-			require.NoError(t, err)
-			_, err = cred.GetToken(context.Background(), testTRO)
-			require.NoError(t, err)
-			_, err = cred.GetToken(ctx, testTRO)
-			require.NoError(t, err)
-			require.Equal(t, 1, tokenReqs)
-
-			// cred2 should return the token cached by cred
-			cred2, err := credential.new(policy.ClientOptions{Transport: &sts}, c)
-			require.NoError(t, err)
-			_, err = cred2.GetToken(ctx, testTRO)
-			require.NoError(t, err)
-			require.Equal(t, 1, tokenReqs)
-
-			// cred should request a new token because the cached one isn't a CAE token
-			caeTRO := testTRO
-			caeTRO.EnableCAE = true
-			_, err = cred.GetToken(ctx, caeTRO)
-			require.NoError(t, err)
-			require.Equal(t, 2, tokenReqs)
-		})
+			if test.options != nil {
+				if test.options.AllowUnencryptedStorage != o.AllowUnencryptedStorage {
+					t.Fatalf("expected AllowUnencryptedStorage %v, got %v", test.options.AllowUnencryptedStorage, o.AllowUnencryptedStorage)
+				}
+				if test.options.Name != o.Name {
+					t.Fatalf("expected Name %q, got %q", test.options.Name, o.Name)
+				}
+			}
+			return nil, nil
+		}
+		for _, subtest := range []struct {
+			ctor func(azcore.ClientOptions, *tokenCachePersistenceOptions) (azcore.TokenCredential, error)
+			env  map[string]string
+			name string
+		}{
+			{
+				name: credNameAssertion,
+				ctor: func(co azcore.ClientOptions, tco *tokenCachePersistenceOptions) (azcore.TokenCredential, error) {
+					o := ClientAssertionCredentialOptions{ClientOptions: co, tokenCachePersistenceOptions: tco}
+					return NewClientAssertionCredential(fakeTenantID, fakeClientID, func(context.Context) (string, error) { return "...", nil }, &o)
+				},
+			},
+			{
+				name: credNameCert,
+				ctor: func(co azcore.ClientOptions, tco *tokenCachePersistenceOptions) (azcore.TokenCredential, error) {
+					o := ClientCertificateCredentialOptions{ClientOptions: co, tokenCachePersistenceOptions: tco}
+					return NewClientCertificateCredential(fakeTenantID, fakeClientID, allCertTests[0].certs, allCertTests[0].key, &o)
+				},
+			},
+			{
+				name: credNameDeviceCode,
+				ctor: func(co azcore.ClientOptions, tco *tokenCachePersistenceOptions) (azcore.TokenCredential, error) {
+					o := DeviceCodeCredentialOptions{
+						ClientOptions:                co,
+						tokenCachePersistenceOptions: tco,
+						UserPrompt:                   func(context.Context, DeviceCodeMessage) error { return nil },
+					}
+					return NewDeviceCodeCredential(&o)
+				},
+			},
+			{
+				name: credNameSecret,
+				ctor: func(co azcore.ClientOptions, tco *tokenCachePersistenceOptions) (azcore.TokenCredential, error) {
+					o := ClientSecretCredentialOptions{ClientOptions: co, tokenCachePersistenceOptions: tco}
+					return NewClientSecretCredential(fakeTenantID, fakeClientID, fakeSecret, &o)
+				},
+			},
+			{
+				name: credNameUserPassword,
+				ctor: func(co azcore.ClientOptions, tco *tokenCachePersistenceOptions) (azcore.TokenCredential, error) {
+					o := UsernamePasswordCredentialOptions{ClientOptions: co, tokenCachePersistenceOptions: tco}
+					return NewUsernamePasswordCredential(fakeTenantID, fakeClientID, fakeUsername, "password", &o)
+				},
+			},
+		} {
+			t.Run(fmt.Sprintf("%s/%s", subtest.name, test.desc), func(t *testing.T) {
+				for k, v := range subtest.env {
+					t.Setenv(k, v)
+				}
+				c, err := subtest.ctor(policy.ClientOptions{Transport: &mockSTS{}}, test.options)
+				if err != nil {
+					t.Fatal(err)
+				}
+				_, err = c.GetToken(context.Background(), testTRO)
+				if err != nil {
+					if !errors.Is(err, test.err) {
+						t.Fatalf("expected %v, got %v", test.err, err)
+					}
+				} else if test.err != nil {
+					t.Fatal("expected an error")
+				}
+			})
+		}
 	}
 }
 
