@@ -30,6 +30,10 @@ var (
 			},
 			InstanceSizeSlug: "professional-xs",
 			InstanceCount:    1,
+			Termination: &AppServiceSpecTermination{
+				GracePeriodSeconds: 100,
+				DrainSeconds:       60,
+			},
 		}},
 		Workers: []*AppWorkerSpec{{
 			Name:           "worker-name",
@@ -42,6 +46,9 @@ var (
 			},
 			InstanceSizeSlug: "professional-xs",
 			InstanceCount:    1,
+			Termination: &AppWorkerSpecTermination{
+				GracePeriodSeconds: 100,
+			},
 		}},
 		StaticSites: []*AppStaticSiteSpec{{
 			Name:         "static-name",
@@ -63,6 +70,9 @@ var (
 			},
 			InstanceSizeSlug: "professional-xs",
 			InstanceCount:    1,
+			Termination: &AppJobSpecTermination{
+				GracePeriodSeconds: 100,
+			},
 		}},
 		Databases: []*AppDatabaseSpec{{
 			Name:        "db",
@@ -335,11 +345,12 @@ func TestApps_UpdateApp(t *testing.T) {
 		err := json.NewDecoder(r.Body).Decode(&req)
 		require.NoError(t, err)
 		assert.Equal(t, &updatedSpec, req.Spec)
+		assert.True(t, req.UpdateAllSourceVersions)
 
 		json.NewEncoder(w).Encode(&appRoot{App: &testApp})
 	})
 
-	app, _, err := client.Apps.Update(ctx, testApp.ID, &AppUpdateRequest{Spec: &updatedSpec})
+	app, _, err := client.Apps.Update(ctx, testApp.ID, &AppUpdateRequest{Spec: &updatedSpec, UpdateAllSourceVersions: true})
 	require.NoError(t, err)
 	assert.Equal(t, &testApp, app)
 }
@@ -416,6 +427,46 @@ func TestApps_ProposeApp(t *testing.T) {
 	assert.True(t, res.AppNameAvailable)
 }
 
+func TestApps_Restart(t *testing.T) {
+	for _, tc := range []struct {
+		name       string
+		components []string
+	}{
+		{
+			name:       "all",
+			components: nil,
+		},
+		{
+			name:       "specific",
+			components: []string{"service1", "service2"},
+		},
+	} {
+		t.Run(tc.name, func(t *testing.T) {
+			setup()
+			defer teardown()
+
+			ctx := context.Background()
+
+			mux.HandleFunc(fmt.Sprintf("/v2/apps/%s/restart", testApp.ID), func(w http.ResponseWriter, r *http.Request) {
+				testMethod(t, r, http.MethodPost)
+
+				var req AppRestartRequest
+				err := json.NewDecoder(r.Body).Decode(&req)
+				require.NoError(t, err)
+				assert.Equal(t, tc.components, req.Components)
+
+				json.NewEncoder(w).Encode(&deploymentRoot{Deployment: &testDeployment})
+			})
+
+			deployment, _, err := client.Apps.Restart(ctx, testApp.ID, &AppRestartRequest{
+				Components: tc.components,
+			})
+			require.NoError(t, err)
+			assert.Equal(t, &testDeployment, deployment)
+		})
+	}
+}
+
 func TestApps_CreateDeployment(t *testing.T) {
 	for _, forceBuild := range []bool{true, false} {
 		t.Run(fmt.Sprintf("ForceBuild_%t", forceBuild), func(t *testing.T) {
@@ -505,6 +556,38 @@ func TestApps_GetLogs(t *testing.T) {
 	assert.NotEmpty(t, logs.LiveURL)
 }
 
+func TestApps_GetExec(t *testing.T) {
+	setup()
+	defer teardown()
+
+	ctx := context.Background()
+
+	mux.HandleFunc(fmt.Sprintf("/v2/apps/%s/deployments/%s/components/%s/exec", testApp.ID, testDeployment.ID, "service-name"), func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, http.MethodGet)
+
+		_, hasComponent := r.URL.Query()["component_name"]
+		assert.False(t, hasComponent)
+
+		json.NewEncoder(w).Encode(&AppExec{URL: "https://exec.url1"})
+	})
+	mux.HandleFunc(fmt.Sprintf("/v2/apps/%s/components/%s/exec", testApp.ID, "service-name"), func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, http.MethodGet)
+
+		_, hasComponent := r.URL.Query()["component_name"]
+		assert.False(t, hasComponent)
+
+		json.NewEncoder(w).Encode(&AppExec{URL: "https://exec.url2"})
+	})
+
+	exec, _, err := client.Apps.GetExec(ctx, testApp.ID, testDeployment.ID, "service-name")
+	require.NoError(t, err)
+	assert.Equal(t, "https://exec.url1", exec.URL)
+
+	exec, _, err = client.Apps.GetExec(ctx, testApp.ID, "", "service-name")
+	require.NoError(t, err)
+	assert.Equal(t, "https://exec.url2", exec.URL)
+}
+
 func TestApps_GetLogs_ActiveDeployment(t *testing.T) {
 	setup()
 	defer teardown()
@@ -933,6 +1016,7 @@ func TestApps_Interfaces(t *testing.T) {
 			&GitHubSourceSpec{},
 			&GitLabSourceSpec{},
 			&ImageSourceSpec{},
+			&BitbucketSourceSpec{},
 		} {
 			if _, ok := impl.(SourceSpec); !ok {
 				t.Fatalf("%T should match interface", impl)
@@ -945,17 +1029,18 @@ func TestApps_Interfaces(t *testing.T) {
 			&GitSourceSpec{},
 			&GitHubSourceSpec{},
 			&GitLabSourceSpec{},
+			&BitbucketSourceSpec{},
 		} {
 			if _, ok := impl.(VCSSourceSpec); !ok {
 				t.Fatalf("%T should match interface", impl)
 			}
 		}
 		for impl, wantMatch := range map[any]bool{
-			&GitSourceSpec{}:    true,
-			&GitHubSourceSpec{}: true,
-			&GitLabSourceSpec{}: true,
-
-			&ImageSourceSpec{}: false,
+			&GitSourceSpec{}:       true,
+			&GitHubSourceSpec{}:    true,
+			&GitLabSourceSpec{}:    true,
+			&BitbucketSourceSpec{}: true,
+			&ImageSourceSpec{}:     false,
 		} {
 			_, ok := impl.(VCSSourceSpec)
 			if wantMatch && !ok {
