@@ -21,7 +21,7 @@ import (
 )
 
 const (
-	libraryVersion = "1.113.0"
+	libraryVersion = "1.132.0"
 	defaultBaseURL = "https://api.digitalocean.com/"
 	userAgent      = "godo/" + libraryVersion
 	mediaType      = "application/json"
@@ -54,41 +54,45 @@ type Client struct {
 	ratemtx sync.Mutex
 
 	// Services used for communicating with the API
-	Account           AccountService
-	Actions           ActionsService
-	Apps              AppsService
-	Balance           BalanceService
-	BillingHistory    BillingHistoryService
-	CDNs              CDNService
-	Certificates      CertificatesService
-	Databases         DatabasesService
-	Domains           DomainsService
-	Droplets          DropletsService
-	DropletActions    DropletActionsService
-	Firewalls         FirewallsService
-	FloatingIPs       FloatingIPsService
-	FloatingIPActions FloatingIPActionsService
-	Functions         FunctionsService
-	Images            ImagesService
-	ImageActions      ImageActionsService
-	Invoices          InvoicesService
-	Keys              KeysService
-	Kubernetes        KubernetesService
-	LoadBalancers     LoadBalancersService
-	Monitoring        MonitoringService
-	OneClick          OneClickService
-	Projects          ProjectsService
-	Regions           RegionsService
-	Registry          RegistryService
-	ReservedIPs       ReservedIPsService
-	ReservedIPActions ReservedIPActionsService
-	Sizes             SizesService
-	Snapshots         SnapshotsService
-	Storage           StorageService
-	StorageActions    StorageActionsService
-	Tags              TagsService
-	UptimeChecks      UptimeChecksService
-	VPCs              VPCsService
+	Account             AccountService
+	Actions             ActionsService
+	Apps                AppsService
+	Balance             BalanceService
+	BillingHistory      BillingHistoryService
+	CDNs                CDNService
+	Certificates        CertificatesService
+	Databases           DatabasesService
+	Domains             DomainsService
+	Droplets            DropletsService
+	DropletActions      DropletActionsService
+	DropletAutoscale    DropletAutoscaleService
+	Firewalls           FirewallsService
+	FloatingIPs         FloatingIPsService
+	FloatingIPActions   FloatingIPActionsService
+	Functions           FunctionsService
+	Images              ImagesService
+	ImageActions        ImageActionsService
+	Invoices            InvoicesService
+	Keys                KeysService
+	Kubernetes          KubernetesService
+	LoadBalancers       LoadBalancersService
+	Monitoring          MonitoringService
+	OneClick            OneClickService
+	Projects            ProjectsService
+	Regions             RegionsService
+	Registry            RegistryService
+	Registries          RegistriesService
+	ReservedIPs         ReservedIPsService
+	ReservedIPV6s       ReservedIPV6sService
+	ReservedIPActions   ReservedIPActionsService
+	ReservedIPV6Actions ReservedIPV6ActionsService
+	Sizes               SizesService
+	Snapshots           SnapshotsService
+	Storage             StorageService
+	StorageActions      StorageActionsService
+	Tags                TagsService
+	UptimeChecks        UptimeChecksService
+	VPCs                VPCsService
 
 	// Optional function called after every successful request made to the DO APIs
 	onRequestCompleted RequestCompletionCallback
@@ -275,6 +279,7 @@ func NewClient(httpClient *http.Client) *Client {
 	c.Domains = &DomainsServiceOp{client: c}
 	c.Droplets = &DropletsServiceOp{client: c}
 	c.DropletActions = &DropletActionsServiceOp{client: c}
+	c.DropletAutoscale = &DropletAutoscaleServiceOp{client: c}
 	c.Firewalls = &FirewallsServiceOp{client: c}
 	c.FloatingIPs = &FloatingIPsServiceOp{client: c}
 	c.FloatingIPActions = &FloatingIPActionsServiceOp{client: c}
@@ -290,8 +295,11 @@ func NewClient(httpClient *http.Client) *Client {
 	c.Projects = &ProjectsServiceOp{client: c}
 	c.Regions = &RegionsServiceOp{client: c}
 	c.Registry = &RegistryServiceOp{client: c}
+	c.Registries = &RegistriesServiceOp{client: c}
 	c.ReservedIPs = &ReservedIPsServiceOp{client: c}
+	c.ReservedIPV6s = &ReservedIPV6sServiceOp{client: c}
 	c.ReservedIPActions = &ReservedIPActionsServiceOp{client: c}
+	c.ReservedIPV6Actions = &ReservedIPV6ActionsServiceOp{client: c}
 	c.Sizes = &SizesServiceOp{client: c}
 	c.Snapshots = &SnapshotsServiceOp{client: c}
 	c.Storage = &StorageServiceOp{client: c}
