@@ -13,11 +13,15 @@ const (
 	loadBalancersBasePath = "/v2/load_balancers"
 )
 
-// Load Balancer types.
 const (
+	// Load Balancer types
 	LoadBalancerTypeGlobal          = "GLOBAL"
 	LoadBalancerTypeRegional        = "REGIONAL"
 	LoadBalancerTypeRegionalNetwork = "REGIONAL_NETWORK"
+
+	// Load Balancer network types
+	LoadBalancerNetworkTypeExternal = "EXTERNAL"
+	LoadBalancerNetworkTypeInternal = "INTERNAL"
 )
 
 // LoadBalancersService is an interface for managing load balancers with the DigitalOcean API.
@@ -41,6 +45,7 @@ type LoadBalancer struct {
 	ID   string `json:"id,omitempty"`
 	Name string `json:"name,omitempty"`
 	IP   string `json:"ip,omitempty"`
+	IPv6 string `json:"ipv6,omitempty"`
 	// SizeSlug is mutually exclusive with SizeUnit. Only one should be specified
 	SizeSlug string `json:"size,omitempty"`
 	// SizeUnit is mutually exclusive with SizeSlug. Only one should be specified
@@ -68,6 +73,7 @@ type LoadBalancer struct {
 	Domains                      []*LBDomain      `json:"domains,omitempty"`
 	GLBSettings                  *GLBSettings     `json:"glb_settings,omitempty"`
 	TargetLoadBalancerIDs        []string         `json:"target_load_balancer_ids,omitempty"`
+	Network                      string           `json:"network,omitempty"`
 }
 
 // String creates a human-readable description of a LoadBalancer.
@@ -101,6 +107,7 @@ func (l LoadBalancer) AsRequest() *LoadBalancerRequest {
 		ProjectID:                    l.ProjectID,
 		HTTPIdleTimeoutSeconds:       l.HTTPIdleTimeoutSeconds,
 		TargetLoadBalancerIDs:        append([]string(nil), l.TargetLoadBalancerIDs...),
+		Network:                      l.Network,
 	}
 
 	if l.DisableLetsEncryptDNSRecords != nil {
@@ -164,6 +171,7 @@ type HealthCheck struct {
 	ResponseTimeoutSeconds int    `json:"response_timeout_seconds,omitempty"`
 	HealthyThreshold       int    `json:"healthy_threshold,omitempty"`
 	UnhealthyThreshold     int    `json:"unhealthy_threshold,omitempty"`
+	ProxyProtocol          *bool  `json:"proxy_protocol,omitempty"`
 }
 
 // String creates a human-readable description of a HealthCheck.
@@ -238,6 +246,7 @@ type LoadBalancerRequest struct {
 	Domains                      []*LBDomain      `json:"domains,omitempty"`
 	GLBSettings                  *GLBSettings     `json:"glb_settings,omitempty"`
 	TargetLoadBalancerIDs        []string         `json:"target_load_balancer_ids,omitempty"`
+	Network                      string           `json:"network,omitempty"`
 }
 
 // String creates a human-readable description of a LoadBalancerRequest.
@@ -290,6 +299,10 @@ type GLBSettings struct {
 	TargetPort uint32 `json:"target_port"`
 	// CDNSettings is the CDN configurations
 	CDN *CDNSettings `json:"cdn"`
+	// RegionPriorities embeds regional priority information for regional active-passive failover policy
+	RegionPriorities map[string]uint32 `json:"region_priorities,omitempty"`
+	// FailoverThreshold embeds failover threshold percentage for regional active-passive failover policy
+	FailoverThreshold uint32 `json:"failover_threshold,omitempty"`
 }
 
 // String creates a human-readable description of a GLBSettings
@@ -299,8 +312,10 @@ func (s GLBSettings) String() string {
 
 func (s GLBSettings) deepCopy() *GLBSettings {
 	settings := &GLBSettings{
-		TargetProtocol: s.TargetProtocol,
-		TargetPort:     s.TargetPort,
+		TargetProtocol:    s.TargetProtocol,
+		TargetPort:        s.TargetPort,
+		RegionPriorities:  s.RegionPriorities,
+		FailoverThreshold: s.FailoverThreshold,
 	}
 	if s.CDN != nil {
 		settings.CDN = &CDNSettings{IsEnabled: s.CDN.IsEnabled}
