@@ -600,7 +600,7 @@ func TestServerPrivateNetSchema(t *testing.T) {
 func TestServerTypeSchema(t *testing.T) {
 	data := []byte(`{
 		"id": 1,
-		"name": "cx10",
+		"name": "cx22",
 		"description": "description",
 		"cores": 4,
 		"memory": 1.0,
@@ -1508,7 +1508,7 @@ func TestLoadBalancerCreateOptsToSchema(t *testing.T) {
 			},
 			Request: schema.LoadBalancerCreateRequest{
 				Name:             "test",
-				LoadBalancerType: "lb11",
+				LoadBalancerType: schema.IDOrName{Name: "lb11"},
 				Algorithm: &schema.LoadBalancerCreateRequestAlgorithm{
 					Type: string(LoadBalancerAlgorithmTypeRoundRobin),
 				},
@@ -1593,7 +1593,7 @@ func TestLoadBalancerCreateOptsToSchema(t *testing.T) {
 			},
 			Request: schema.LoadBalancerCreateRequest{
 				Name:             "test",
-				LoadBalancerType: "lb11",
+				LoadBalancerType: schema.IDOrName{Name: "lb11"},
 				Algorithm: &schema.LoadBalancerCreateRequestAlgorithm{
 					Type: string(LoadBalancerAlgorithmTypeRoundRobin),
 				},
