@@ -9,6 +9,7 @@ import (
 
 	jsoniter "github.com/json-iterator/go"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 
 	otlpmetrics "go.opentelemetry.io/collector/pdata/internal/data/protogen/metrics/v1"
 	"go.opentelemetry.io/collector/pdata/pcommon"
@@ -21,20 +22,22 @@ var metricsOTLP = func() Metrics {
 	il := rm.ScopeMetrics().AppendEmpty()
 	il.Scope().SetName("name")
 	il.Scope().SetVersion("version")
-	il.Metrics().AppendEmpty().SetName("testMetric")
+	m := il.Metrics().AppendEmpty()
+	m.SetName("testMetric")
+	m.Metadata().PutStr("metadatakey", "metadatavalue")
 	return md
 }()
 
-var metricsJSON = `{"resourceMetrics":[{"resource":{"attributes":[{"key":"host.name","value":{"stringValue":"testHost"}}]},"scopeMetrics":[{"scope":{"name":"name","version":"version"},"metrics":[{"name":"testMetric"}]}]}]}`
+var metricsJSON = `{"resourceMetrics":[{"resource":{"attributes":[{"key":"host.name","value":{"stringValue":"testHost"}}]},"scopeMetrics":[{"scope":{"name":"name","version":"version"},"metrics":[{"name":"testMetric","metadata":[{"key":"metadatakey","value":{"stringValue":"metadatavalue"}}]}]}]}]}`
 
 func TestMetricsJSON(t *testing.T) {
 	encoder := &JSONMarshaler{}
 	jsonBuf, err := encoder.MarshalMetrics(metricsOTLP)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	decoder := &JSONUnmarshaler{}
 	got, err := decoder.UnmarshalMetrics(jsonBuf)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	assert.EqualValues(t, metricsOTLP, got)
 }
@@ -42,8 +45,8 @@ func TestMetricsJSON(t *testing.T) {
 func TestMetricsJSON_Marshal(t *testing.T) {
 	encoder := &JSONMarshaler{}
 	jsonBuf, err := encoder.MarshalMetrics(metricsOTLP)
-	assert.NoError(t, err)
-	assert.Equal(t, metricsJSON, string(jsonBuf))
+	require.NoError(t, err)
+	assert.JSONEq(t, metricsJSON, string(jsonBuf))
 }
 
 var metricsSumOTLPFull = func() Metrics {
@@ -300,10 +303,10 @@ func Test_jsonUnmarshaler_UnmarshalMetrics(t *testing.T) {
 			encoder := &JSONMarshaler{}
 			m := tt.args.md()
 			jsonBuf, err := encoder.MarshalMetrics(m)
-			assert.NoError(t, err)
+			require.NoError(t, err)
 			decoder := JSONUnmarshaler{}
 			got, err := decoder.UnmarshalMetrics(jsonBuf)
-			assert.NoError(t, err)
+			require.NoError(t, err)
 			assert.EqualValues(t, m, got)
 		})
 	}
@@ -315,7 +318,7 @@ func TestUnmarshalJsoniterMetricsData(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewMetrics()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, NewMetrics(), val)
 }
 
@@ -325,7 +328,7 @@ func TestUnmarshalJsoniterResourceMetrics(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewResourceMetrics()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, &otlpmetrics.ResourceMetrics{SchemaUrl: "schema"}, val.orig)
 }
 
@@ -335,7 +338,7 @@ func TestUnmarshalJsoniterScopeMetrics(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewScopeMetrics()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, &otlpmetrics.ScopeMetrics{SchemaUrl: "schema"}, val.orig)
 }
 
@@ -416,7 +419,7 @@ func TestUnmarshalJsoniterMetric(t *testing.T) {
 		jsoniter.ConfigFastest.ReturnIterator(iter)
 		val := NewMetric()
 		val.unmarshalJsoniter(iter)
-		assert.NoError(t, iter.Error)
+		require.NoError(t, iter.Error)
 		assert.EqualValues(t, tt.args.want, val.orig)
 	}
 }
@@ -427,7 +430,7 @@ func TestUnmarshalJsoniterNumberDataPoint(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewNumberDataPoint()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, NewNumberDataPoint(), val)
 }
 
@@ -437,7 +440,7 @@ func TestUnmarshalJsoniterHistogramDataPoint(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewHistogramDataPoint()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, &otlpmetrics.HistogramDataPoint{Count: 3}, val.orig)
 }
 
@@ -447,7 +450,7 @@ func TestUnmarshalJsoniterExponentialHistogramDataPoint(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewExponentialHistogramDataPoint()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, &otlpmetrics.ExponentialHistogramDataPoint{Count: 3}, val.orig)
 }
 
@@ -457,7 +460,7 @@ func TestUnmarshalJsoniterExponentialHistogramDataPointBuckets(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewExponentialHistogramDataPointBuckets()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, &otlpmetrics.ExponentialHistogramDataPoint_Buckets{Offset: 3, BucketCounts: []uint64{1, 2}}, val.orig)
 }
 
@@ -467,7 +470,7 @@ func TestUnmarshalJsoniterSummaryDataPoint(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewSummaryDataPoint()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, &otlpmetrics.SummaryDataPoint{
 		Count: 3,
 		Sum:   3.14,
@@ -480,7 +483,7 @@ func TestUnmarshalJsoniterQuantileValue(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewSummaryDataPointValueAtQuantile()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, &otlpmetrics.SummaryDataPoint_ValueAtQuantile{
 		Quantile: 0.314,
 		Value:    3,
@@ -528,9 +531,7 @@ func TestExemplarInvalidTraceID(t *testing.T) {
 	iter := jsoniter.ConfigFastest.BorrowIterator([]byte(jsonStr))
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	NewExemplar().unmarshalJsoniter(iter)
-	if assert.Error(t, iter.Error) {
-		assert.Contains(t, iter.Error.Error(), "parse trace_id")
-	}
+	assert.ErrorContains(t, iter.Error, "parse trace_id")
 }
 
 func TestExemplarInvalidSpanID(t *testing.T) {
@@ -538,9 +539,7 @@ func TestExemplarInvalidSpanID(t *testing.T) {
 	iter := jsoniter.ConfigFastest.BorrowIterator([]byte(jsonStr))
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	NewExemplar().unmarshalJsoniter(iter)
-	if assert.Error(t, iter.Error) {
-		assert.Contains(t, iter.Error.Error(), "parse span_id")
-	}
+	assert.ErrorContains(t, iter.Error, "parse span_id")
 }
 
 func TestExemplar(t *testing.T) {
@@ -549,6 +548,6 @@ func TestExemplar(t *testing.T) {
 	defer jsoniter.ConfigFastest.ReturnIterator(iter)
 	val := NewExemplar()
 	val.unmarshalJsoniter(iter)
-	assert.NoError(t, iter.Error)
+	require.NoError(t, iter.Error)
 	assert.EqualValues(t, NewExemplar(), val)
 }
