@@ -7,7 +7,6 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
-	"math/rand"
 	"net/http"
 	"runtime"
 	"time"
@@ -15,7 +14,7 @@ import (
 	"go.opentelemetry.io/otel"
 	"go.opentelemetry.io/otel/attribute"
 	"go.opentelemetry.io/otel/metric"
-	semconv "go.opentelemetry.io/otel/semconv/v1.26.0"
+	semconv "go.opentelemetry.io/otel/semconv/v1.25.0"
 )
 
 var meter = otel.Meter("my-service-meter")
@@ -143,45 +142,6 @@ func ExampleMeter_upDownCounter() {
 	}
 }
 
-// Gauges can be used to record non-additive values when changes occur.
-//
-// Here's how you might report the current speed of a cpu fan.
-func ExampleMeter_gauge() {
-	speedGauge, err := meter.Int64Gauge(
-		"cpu.fan.speed",
-		metric.WithDescription("Speed of CPU fan"),
-		metric.WithUnit("RPM"),
-	)
-	if err != nil {
-		panic(err)
-	}
-
-	getCPUFanSpeed := func() int64 {
-		// Generates a random fan speed for demonstration purpose.
-		// In real world applications, replace this to get the actual fan speed.
-		return int64(1500 + rand.Intn(1000))
-	}
-
-	fanSpeedSubscription := make(chan int64, 1)
-	go func() {
-		defer close(fanSpeedSubscription)
-
-		for idx := 0; idx < 5; idx++ {
-			// Synchronous gauges are used when the measurement cycle is
-			// synchronous to an external change.
-			// Simulate that external cycle here.
-			time.Sleep(time.Duration(rand.Intn(3)) * time.Second)
-			fanSpeed := getCPUFanSpeed()
-			fanSpeedSubscription <- fanSpeed
-		}
-	}()
-
-	ctx := context.Background()
-	for fanSpeed := range fanSpeedSubscription {
-		speedGauge.Record(ctx, fanSpeed)
-	}
-}
-
 // Histograms are used to measure a distribution of values over time.
 //
 // Here's how you might report a distribution of response times for an HTTP handler.
@@ -232,7 +192,7 @@ func ExampleMeter_observableUpDownCounter() {
 	// The function registers asynchronous metrics for the provided db.
 	// Make sure to unregister metric.Registration before closing the provided db.
 	_ = func(db *sql.DB, meter metric.Meter, poolName string) (metric.Registration, error) {
-		m, err := meter.Int64ObservableUpDownCounter(
+		max, err := meter.Int64ObservableUpDownCounter(
 			"db.client.connections.max",
 			metric.WithDescription("The maximum number of open connections allowed."),
 			metric.WithUnit("{connection}"),
@@ -253,11 +213,11 @@ func ExampleMeter_observableUpDownCounter() {
 		reg, err := meter.RegisterCallback(
 			func(_ context.Context, o metric.Observer) error {
 				stats := db.Stats()
-				o.ObserveInt64(m, int64(stats.MaxOpenConnections))
+				o.ObserveInt64(max, int64(stats.MaxOpenConnections))
 				o.ObserveInt64(waitTime, int64(stats.WaitDuration))
 				return nil
 			},
-			m,
+			max,
 			waitTime,
 		)
 		if err != nil {
